/** 
	A program in Qt that use OpenGL drawing primitives

	Created to show the animation of rigid structures

	File: robot.cpp

	Fraga 22/07/2003
**/
#include <qpainter.h>
#include <math.h>
#include "robot.h"

Robot::Robot( QWidget *parent, const char *name )
        : QGLWidget( parent, name )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	// setDoubleBuffer ( TRUE );  // This is enable by default
	// setRgba( TRUE );           // This is enable by default
	// setDepth( TRUE );          // This is enable by default

	h= 0.3;
	b = 5.5;
	c = 3.6;

	betamax = 150;
	betamin =  10;

	gamma = igamma =  90;
	theta = itheta = betamin - 180;

	xval = yval = 0.0f;

	state = 0;

	float tmp = b+c;
	dmax = sqrt(tmp*tmp - h*h);
}


void Robot::timerDone( )
{
	if ( state == 1 ) {
		theta += STEP;
		if ( theta >= ftheta ) {
			theta = ftheta;
			state++;
		}
	}
	else if ( state == 2 ) {
		gamma -= STEP;	
		if ( gamma <= fgamma ) {
			gamma = fgamma;
			state++;
		}
	}
	else {
		timer->stop( );
	}
	
	updateGL( );
}

void Robot::mousePressEvent( QMouseEvent * e )
{
	QPoint m = e->pos( );

	xmouse = m.x( );
	ymouse = m.y( );

	float aspect = (float)windowWidth/(float)windowHeight;

	if ( windowWidth > windowHeight ) {
		//  Height is bounden by MIN and MAX
    	yval = windowHeight - ymouse; // Changing x axis sense
    	yval = yval*(MAX-MIN)/(float)windowHeight + MIN;

    	xval = aspect*((float)xmouse*(MAX-MIN)/(float)windowWidth + MIN);
	}
	else {
		//  Width is bounded by MIN and MAX
    	xval = (float)xmouse*(MAX-MIN)/(float)windowWidth + MIN;

		yval = ((float)ymouse*(MAX-MIN)/(float)windowHeight + MIN)/aspect;
	}

	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	// We going to ckeck if we can do the animation
	if ( ! state ) {
		if ( fabs(yval) < 0.05 ) {
			state++;
			// Ok. ( xval, 0 ) is a posible final point	
			// to move the robot "eye"

			if ( xval > dmax ) { // we cannot move the robot!	
				qWarning( "Cannot move robot to that point\n" 
				          "(%2.3f,%2.3f)\n", xval, yval );
				state=0;
				return; 
			}

			float a = sqrt( xval*xval + h*h);

			float cosbeta  = ( -a*a + b*b + c*c )/(2*b*c);
			float cosgamma = (  a*a + b*b - c*c )/(2*a*b);

			float tmp1 = fabs( cosbeta );
			float tmp2 = fabs( cosgamma );

			if ( tmp1>=0.0 && tmp1<=1.0 && tmp2>=0 && tmp2<=1.0 ) {
				ftheta = acos ( cosbeta );
				fgamma = acos ( cosgamma );

				ftheta = ftheta*180/3.141592654 - 180.0f;
				fgamma = fgamma*180/3.141592654;

				a = atan( h/xval );
				a = a*180/3.141592654;

				fgamma = fgamma - a;

				startAnimation( );
			}
			else {
				qWarning( "Cannot move robot to that point\n" 
				          "(%2.3f,%2.3f)\n", xval, yval );
				state=0;
				return; 
			}
		} 
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Robot::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	// glOrtho (-aspect, aspect, -1.0, 1.0, -1.0, 1.0);
	if ( w > h ) 
		glOrtho ( MIN*aspect, MAX*aspect, MIN, MAX, -1.0, 1.0);
	else
		glOrtho ( MIN, MAX, MIN*aspect, MAX*aspect, -1.0, 1.0);

	emit changeSize ( );
}

QSizePolicy Robot::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Robot::paintGL(void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */

	/* viewing transformation  */
//	glPushMatrix( );
//	glScalef ( 0.3, 0.3, 0.3);      /* modeling transformation */ 

	// Dibujo la base
	glBegin(GL_POLYGON);
		glColor3f( 1.0, 0.62, 0.19 );
		glVertex3f( -1.0, 0.0, 0.0 );
		glVertex3f( 10.0, 0.0, 0.0 );
		glVertex3f( 10.0,-0.3, 0.0 );
		glVertex3f( -1.0,-0.3, 0.0 );
	glEnd();

	glTranslatef( 0.0, h, 0.0);

	// Dibujo el antebrazo	
	glRotatef ( gamma, 0.0, 0.0, 1.0);
	glBegin(GL_LINES);
		glColor3f( 0.0, 0.0, 1.0 );
		glVertex3f( 0.0, 0.0, 0.0 );
		glVertex3f( b, 0.0, 0.0 );
	glEnd();

	// Dibujo el brazo
	// glPushMatrix( );

	glTranslatef( b, 0.0, 0.0);
	glRotatef ( theta, 0.0, 0.0, 1.0);

	glBegin(GL_LINES);
		glColor3f( 1.0, 0.0, 0.0 );
		glVertex3f( 0.0, 0.0, 0.0 );
		glVertex3f( c, 0.0, 0.0 );
	glEnd();

	// glPopMatrix( );

	swapBuffers();
}

void Robot::initializeGL( ) 
{
	glClearColor (0.8, 0.8, 0.8, 0.0);	// Background to white

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho( MIN, MAX, MIN, MAX, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);
}

